@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  parsers.c - Slurm data parsers
  *****************************************************************************
- *  Copyright (C) 2022 SchedMD LLC.
- *  Written by Nathan Rini <nate@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -545,6 +544,15 @@ static int _parser_linked(args_t *args, const parser_t *const array,
 		openapi_append_rel_path(ppath, parser->key);
 	}
 
+	if (parser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD) {
+		log_flag(DATA, "%s: skip parsing removed %s object %s(0x%" PRIxPTR ") via parser %s(0x%" PRIxPTR ")",
+			__func__, set_source_path(&path, ppath),
+			parser->obj_type_string, (uintptr_t) dst,
+			parser->type_string, (uintptr_t) src);
+		rc = SLURM_SUCCESS;
+		goto cleanup;
+	}
+
 	if (!src) {
 		if (parser->required) {
 			if ((rc = on_error(PARSING, parser->type, args,
@@ -760,6 +768,9 @@ extern int parse(void *dst, ssize_t dst_bytes, const parser_t *const parser,
 	case PARSER_MODEL_ARRAY_SKIP_FIELD:
 		fatal_abort("%s: skip model not allowed %u",
 			    __func__, parser->model);
+	case PARSER_MODEL_ARRAY_REMOVED_FIELD:
+		fatal_abort("%s: removed model not allowed %u",
+			    __func__, parser->model);
 	case PARSER_MODEL_INVALID:
 	case PARSER_MODEL_MAX:
 		fatal_abort("%s: invalid model %u", __func__, parser->model);
@@ -1085,6 +1096,60 @@ static int _dump_linked(args_t *args, const parser_t *const array,
 		goto cleanup;
 	}
 
+	if (parser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD) {
+		const parser_t *rparser = find_parser_by_type(parser->type);
+
+		while ((rparser->model == PARSER_MODEL_ARRAY_REMOVED_FIELD) ||
+		       rparser->pointer_type) {
+			while (rparser->pointer_type)
+				rparser = find_parser_by_type(
+					rparser->pointer_type);
+
+			while (rparser->model ==
+			       PARSER_MODEL_ARRAY_REMOVED_FIELD)
+				rparser = find_parser_by_type(rparser->type);
+		}
+
+		log_flag(DATA, "removed: %s parser %s->%s(0x%" PRIxPTR ") for %s(0x%" PRIxPTR ") for data(0x%" PRIxPTR ")/%s(0x%" PRIxPTR ")",
+			 parser->obj_type_string,
+			 array->type_string,
+			 parser->type_string, (uintptr_t)
+			 parser, array->obj_type_string,
+			 (uintptr_t) src, (uintptr_t) dst,
+			 array->key, (uintptr_t) dst);
+
+		switch (rparser->obj_openapi) {
+			case OPENAPI_FORMAT_INT:
+			case OPENAPI_FORMAT_INT32:
+			case OPENAPI_FORMAT_INT64:
+				data_set_int(dst, 0);
+				break;
+			case OPENAPI_FORMAT_NUMBER:
+			case OPENAPI_FORMAT_FLOAT:
+			case OPENAPI_FORMAT_DOUBLE:
+				data_set_float(dst, 0);
+				break;
+			case OPENAPI_FORMAT_STRING:
+			case OPENAPI_FORMAT_PASSWORD:
+				data_set_string(dst, "");
+				break;
+			case OPENAPI_FORMAT_BOOL:
+				data_set_bool(dst, false);
+			case OPENAPI_FORMAT_OBJECT:
+				data_set_dict(dst);
+				break;
+			case OPENAPI_FORMAT_ARRAY:
+				data_set_list(dst);
+				break;
+			case OPENAPI_FORMAT_MAX:
+			case OPENAPI_FORMAT_INVALID:
+				fatal_abort("invalid type");
+		};
+
+		rc = SLURM_SUCCESS;
+		goto cleanup;
+	}
+
 	xassert(parser->model == PARSER_MODEL_ARRAY_LINKED_FIELD);
 
 	log_flag(DATA, "BEGIN: dumping %s parser %s->%s(0x%" PRIxPTR ") for %s(0x%" PRIxPTR ")->%s(+%zd) for data(0x%" PRIxPTR ")/%s(0x%" PRIxPTR ")",
@@ -1107,19 +1172,6 @@ cleanup:
 	return rc;
 }
 
-static void _check_dump(const parser_t *const parser, data_t *dst, args_t *args)
-{
-	/*
-	 * Resultant dump must be the proscribed OpenAPI compatible data_t type.
-	 * Anything else will break most generated OpenAPI clients.
-	 */
-	if (parser->obj_openapi == OPENAPI_FORMAT_INVALID)
-		return;
-
-	xassert(data_get_type(dst) ==
-		openapi_type_format_to_data_type(parser->obj_openapi));
-}
-
 extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
 		data_t *dst, args_t *args)
 {
@@ -1207,7 +1259,6 @@ extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
 		 */
 
 		rc = parser->dump(parser, src, dst, args);
-		_check_dump(parser, dst, args);
 		break;
 	case PARSER_MODEL_ARRAY_LINKED_FIELD:
 		fatal_abort("%s: link model not allowed %u",
@@ -1215,6 +1266,9 @@ extern int dump(void *src, ssize_t src_bytes, const parser_t *const parser,
 	case PARSER_MODEL_ARRAY_SKIP_FIELD:
 		fatal_abort("%s: skip model not allowed %u",
 			    __func__, parser->model);
+	case PARSER_MODEL_ARRAY_REMOVED_FIELD:
+		fatal_abort("%s: removed model not allowed %u",
+			    __func__, parser->model);
 	case PARSER_MODEL_INVALID:
 	case PARSER_MODEL_MAX:
 		fatal_abort("%s: invalid model %u", __func__, parser->model);
